#ifndef LOWLEVEL_H
#define LOWLEVEL_H

/* config... */

#define CfgFile "CONFIG.DAT"
#define MaxMs 10

#define NoEncodersStr "NoEncoders"
#define EncoderBSegStr "Bseg"
#define EncoderResetViaRStr "ResetViaR"
#define EncoderResetViaZStr "ResetViaZ"
#define EncoderNoResetStr "NoReset"
#define EncoderMouseStr "Mouse"

/* configuration vars... */

int DefaultBackground;
int DefaultColor;
int TitleColor;
int BorderColor;
int MenuColor;
int DisplayColor;
int SelectColor;
int CurrentColor;
int SelectBackground;
/* if 1, display opening messages */
Flag DisplayOpeningMsgs;
/* move of this distance or greater triggers confirmation msg */
double MoveHsMsgDeg;
/* location where SlewFile and SlewOutFile are located */
char* InterfacePath;
/* if handpad present, then set to 1 */
Flag HandPadPresentFlag;
/* 1=use IACA */
Flag IACA_Flag;
/* move beyond this limits not allowed while slewing using interrupt timer */
/* to disable, set both limits to zero */
double AltLowLimitDeg;
double AltHighLimitDeg;
/* to disable, set both limits to zero */
double AzLowLimitDeg;
double AzHighLimitDeg;
Flag Siderostat;
Flag Equatorial;
double HomeAltDeg;
double HomeAzDeg;
int MsArcsecSec;
int BacklashMsArcsecSec;
double AltFullStepSizeArcsec;
double AzFullStepSizeArcsec;
/* if true, then refraction included in calcs; if false, refraction display
only */
Flag RefractFlag;
double AltBacklashArcmin;
double AzBacklashArcmin;
/* guiding speed */
int GuideArcsecSec;
/* drift to drag a guide star so that an autoguider can  update in one direction
only */
double GuideDragAltArcsecPerMin;
double GuideDragAzArcsecPerMin;
double GuideDragRaArcsecPerMin;
double GuideDragDecArcsecPerMin;
/* set to 1 if using automatic synchronization of alt PEC alignment via parallel
port input triggered by a microswitch */
Flag AutoAltPECSyncOnFlag;
/* if 1, detect low-high transition, if 0, detect high-low transition */
Flag AutoAltPECSyncLowHighFlag;
/* direction that alt PEC must be moving in order to trigger auto-sync:
;0 = either direction, 1 = CCW direction, 2 = CW direction */
Flag AutoAltPECSyncDirFlag;
/* set to 1 if using automatic synchronization of az PEC alignment via parallel
port input triggered by a microswitch */
Flag AutoAzPECSyncOnFlag;
/* if 1, detect low-high transition, if 0, detect high-low transition */
Flag AutoAzPECSyncLowHighFlag;
/* direction that az PEC must be moving in order to trigger auto-sync:
;0 = either direction, 1 = CCW direction, 2 = CW direction */
Flag AutoAzPECSyncDirFlag;
/* PEC cycle set by fullsteps, instead of stepper rotor, so that PEC can
cover double and quad worms plus other reductions */
int FullstepsPerPECArray;
/* field rotation step size in arc seconds; if zero, then field rotation is
ignored */
double FRStepSizeArcsec;
/* set to 1 for a sectored field derotator drive unit, otherwise unit may slew
large angles */
Flag SectoredFRDrive;
/* speed of field rotation motor when slewed by hand paddle */
int FRStepSpeedMilliSec;
/* if 1, then output is inverted, ie, from logical high to logical low */
int InvertOutput;
/* if 1, use IRQ 8 to time halfsteps, otherwise disable IRQs and time
halfsteps with a loop */
int HsTimerFlag;
/* delay loop value for slowest halfstep motor speed */
int MaxDelay;
/* delay loop value for highest halfstep motor speed */
int MinDelay;
/* # of times to repeat delay loop: cuts down on delay array size */
int HsDelayX;
/* halfstep ramp time multiplier */
int HsRampX;
/* # of steps ok to do without disabling IRQs: ignored if HsTimerFlag is 1 */
int InterruptHs;
/* # of repeats of slowest motor speed to hold motors in position */
int HoldReps;
/* max # of consecutive slews before program recalcs Current equatorial based on
Current altazimuth */
int MaxConsecutiveSlews;
/* number of pulse width modulations per bios clock tick */
int PWMRepsTick;
/* reps of PWM[] to form a voltage waveform */
int MsDelayX;
int MsPause;
/* number of microsteps per fullstep; cannot exceed MaxMs */
int Ms;
int PWM[MaxMs];
char* EncoderString;
int EncoderComPort;
int SerialWriteDelayMs;
unsigned AltEncoderCountsPerRev;
unsigned AzEncoderCountsPerRev;
Flag AltEncoderDir;
Flag AzEncoderDir;
double EncoderErrorThresholdDeg;
double TrackEncoderErrorThresholdDeg;
Flag MakeEncoderResetLogFile;
/* comm port to accept LX200 serial protocol commands */
int LX200ComPort;
int LX200MotionTimeoutSec;
long LX200SlewHs;
double Z1Deg;
double Z2Deg;
double Z3Deg;
double LatitudeDeg;
double LongitudeDeg;
int Tz;
int DST;
char* TestString;

void ReadConfig( void);

/* encoders... */

#define MaxEnc 15
#define MaxEncBSeg 15
#define MaxEncStr 14
#define EncoderResetLogFile "ENCODERS.TXT"

struct AZByte
{
	Byte A, Z;
};
struct AZFlag
{
	Flag A, Z;
};
struct AZInt
{
	int A, Z;
};
struct AZUnsigned
{
	unsigned A, Z;
};
struct AZLong
{
	long A, Z;
};
struct AZDouble
{
	double A, Z;
};

typedef enum
{
	NoEncoders,
	EncoderBSeg,
	EncoderResetViaR,
	EncoderResetViaZ,
	EncoderNoReset,
	EncoderMouse,
	MaxEncoderType
}ENCODER_TYPE;

typedef enum
{
	NotInitialized,
	CommError,
	ReadReady,
	NoRead,
	Read
}ENCODER_STATE;

Byte EncStr[MaxEnc];
int EncIx;
ENCODER_TYPE EncoderType;
ENCODER_STATE EncoderState;
struct AZLong EncoderCount;
/* # of times encoders have been reset */
int EncoderTrackOnResetCount;
int EncoderTrackOffResetCount;
int EncoderSlewResetCount;
/* (re)set encoder box at program start up time - set to false if encoder box
has been up and running since last time scope.exe was run */
Flag ResetEncoderBox;
FILE* EncoderOutput;

void( *ResetEncoders_f_ptr) ( void);
void( *ReadEncoders_f_ptr) ( void);
void( *QueryEncoders_f_ptr) ( void);

void InitEncoders( void);
void CloseEncoderResetLogFile( void);
void InitEncodersBSeg( void);
void InitEncodersResetViaR( void);
void InitEncodersResetViaZ( void);
void InitEncodersNoReset( void);
void InitEncodersMouse( void);

void ResetEncodersBSeg( void);
void ResetEncodersResetViaR( void);
void ResetEncodersResetViaZ( void);
void ResetEncodersNoReset( void);
void ResetEncodersMouse( void);
void BuildEncStrRes( void);

void QueryEncodersQ( void);
void QueryEncodersMouse( void);

void ReadEncodersQ( void);
void ReadEncodersBSeg( void);
void ReadEncodersMouse( void);

void QueryAndReadEncoders( void);
void TestEncoders( void);

/* handpad... */

#define UpKey 16
#define DownKey 32
#define CCWKey 64
#define CWKey (16 + 32)
#define LeftKey (16 + 64)
#define RightKey (32 + 64)
#define MsKey 128

/* return value from handpad */
int HandPad;
/* value of HandPad at start of move */
int InitHandPad;
/* if handPad changes during move, HandPadOKFlag set to No */
int HandPadOKFlag;
/* Pad's speed switch setting: if true, speed == microstepping */
int MsSpeed;
/* Pad's buttons */
int Buttons;

void( *ReadHandPad_f_ptr) ( void);

void InitHandpad( void);
void ReadHandPad( void);
void NoHandPad( void);
void SetHandPadOKFlag( void);
void TestHandpad( void);

/* steppers.. */

#define Timer_Int 8
#define Timer_0 0x40
#define Timer_Control 0x43

#define MaxHsIx 7
#define MotorWindings 4

/* must be 2^n - 1 */
#define PWMRepsTickArrayEnd 15

void interrupt( *ClockVect)();

typedef enum
{
	RampUp,
	MaxSlewUp,
	MaxSlewDown,
	RampDown,
	SlewDone
}SLEW_STATE;

typedef enum
{
	NoAbort,
	HandPadAbort,
	KeyStrokeAbort,
	AltLowLimitAbort,
	AltHighLimitAbort,
	AzLowLimitAbort,
	AzHighLimitAbort
}ABORT_STATE;

struct AZByte HsOut[MaxHsIx+1];
/* address of outport port that controls steppers */
unsigned MotorPort;
/* variables that turn off all windings belonging to the stepper motor */
struct AZByte MotorOff;
/* actual loop counts per clock tick */
int ActualPWMRepsTick;
/* steps: must always be positive */
struct AZLong Steps;
struct AZFlag Dir;

/* variables relating to halfstepping... */

double AltLowLimit;
double AltHighLimit;
double AzLowLimit;
double AzHighLimit;
Flag AltLimitFlag;
Flag AzLimitFlag;

/* used to tell Hold() which motors to energize */
struct AZByte HoldMotor;

/* for speed control and linear ramping:
HsReps is array of # of half step repetitions to do for each delay
or timer value: delay value is set by moving through an empty for next
loop HsRepsIx + MinDelay number of times; timer value set by HsRepsIx +
MinDelay; HsRepsIx varies from 0 to MaxDelay - MinDelay; HsReps[0] = MAXLONG; */
long* HsReps;
struct AZInt HsIx;
int HsRepsIx;
int MaxHsRepsIx;
/* max halfsteps that ASteps or ZSteps can be set to */
long MaxHs;
/* # of steps to move */
long MvmtHs;
/* initial # of steps to ramp up */
long RampHs;
/* # of steps to take for a particular HsReps[HsRepsIx] delay or timer value */
long SameSpeedHs;
/* if 1, then don't need IRQs disabled for halfstepping */
Flag InterruptFlag;
unsigned int Timer;
/* # of halfsteps for each stage of halfstep slewing */
long RampUpSteps, MaxSlewUpSteps, MaxSlewDownSteps, RampDownSteps;
SLEW_STATE SlewState;
ABORT_STATE AbortState;

/* variables relating to microstepping... */

int MsPerHs;
/* conversion factor for microstep array indexes to halfstep array indexes */
int HsIxToMsIxConvFactor;
/* accumulated motors' movement stored in form of microsteps */
struct AZLong AccumMs;
/* Ms increment per Microstep PWM;
determines speed of microstepping; # of microsteps per PWM */
struct AZDouble IncrMs;
Flag MsAlignFlag;
int MaxMsIx;
/* microstepping arrays containing on/offs generating voltage values */
struct AZByte* MsPWM;
struct AZInt MsIx;
/* value of passed PWM[0] */
int MaxPWM;
/* microstep distances in arc sec */
struct AZDouble MicroStepSizeArcsec;
/* alt, az microstep sizes in radians */
struct AZDouble MsRad;
/* alt, az halfstep sizes in radians */
struct AZDouble HsRad;
/* max microstepping distance to move in radians per tick */
struct AZDouble MaxMsDistanceRadTick;
/* maximum microstepping speed possible */
int MaxMsSpeed;
/* microsteps per clock tick */
struct AZDouble MsTick;

void InitSteppers( void);
void CreateHsArrays( void);
void InitHsArrays( void);
void FreeHsArrays( void);
void CreateMsArrays( void);
void InitMsArrays( void);
void FreeMsArrays( void);
void CloseSteppers( void);
void InitMsTickVars( const int MsArcsecSec);
void SteppersOff( void);
void Hold( void);
void PauseUntilNewSidTime( void);
void DoOneHs( void);
void DoHsDelayLoop( void);
void SetTickDelay( void);
void interrupt SlewTickHandler( void);
void RoundUpHs( void);
void MoveHs( void);
void MoveHsUsingIRQTimer( void);
void MoveHsUsingDelayLoop( void);
void MoveMs( void);
void AlignMs( void);

/* convert... */

#define AlignFile "ALIGN.DAT"
#define InitFile "INIT.DAT"

/* Conversion routines are based on Sky and Tel, Feb, '89, pg 194-196,
Convert functions input and output horizon increasing CW, then convert to CCW
internally for calculating as per S&T program.
Set coordinates including SidT before making function calls.
Avoid repeatedly alternating GetEquat() and GetAltaz() calls, which can lead to
hysteresis or creep of the coordinates of approx 0.1 arc sec */

/* storage arrays for matrix multiplication */
double QQ[4][4];
double VV[4][4]; 
double RR[4][4]; 
double XX[4][4];
double YY[4][4]; 

/* fabrication errors (in radians): */

/* offset of elevation to perpendicular of horizon, ie, one side of rocker
box higher than the other */
double Z1; 
/* optical axis pointing error in same plane, ie, tube horiz.: optical axis
error left to right (horiz) */
double Z2; 
/* correction to zero setting of elevation, ie, vertical offset error */
double Z3;
/* if Z1, Z2, or Z3 is non-zero */
Flag Z1Z2NonZeroFlag;
Flag SaveInitFlag;
/* coordinates from initialize position #1, and #2 */
struct Position One, Two, Three;
/* following values in radians */
double Lat;
double LongitudeRad;
double SinLat;
double CosLat;
/* this value, also in radians, used by field rotation angle calculation */
double SinLatDividedByCosLat;
/* field rotation angle */
double FieldR;
/* azimuth offset of init'ed position from true azimuth */
double AzOff;
/* Hour Angle Offset = Local Sidereal Time - scope's meridian, or,
HAOff = LST - HA - Ra, or, HA = LST - HAOff - Ra;
(+) offset = scope tilted to West, (-) offset = scope tilted to East;
HAOff varies from - offset to + offset (should only be a few deg.) */
double HAOff;
struct HMSH HAOffHMSH;
double AltOffset;
/* working vars */
double F, H, W;

void InitConvert( void);
void InitConvertEquat( void);
void InitConvertAltaz( void);
void InitConvertEquatMeridianFlip( void);
void ConvertAlignFile( void);
void SaveAlign( void);
Flag LoadAlign( void);
void SaveInit( void);
void SetMountErrorsDeg( const double Z1Deg, const double Z2Deg,
const double Z3Deg);
void ZeroArrays( void);
void ArrayAsignInit( const int Init);
void GenerateThirdInit( void);
void TransformMatrix( void);
void InitMatrix( const int Init);
void SubrA( void);
void SubrB( void);
void DeterminateSubr( void);
void AngleSubr( void);
void CalcPostInitVars( void);
void GetAltaz( void);
void GetEquat( void);
void CalcFieldR( void);
void DisplayFieldR( void);
double CalcEquatAngularSep( struct Position A, struct Position Z);
double CalcAltazAngularSep( struct Position A, struct Position Z);
Flag CalcAltOffset( double* AltOffset, struct Position A, struct Position Z);
void CalcBestAltOffset( void);
void TestConvert( void);
void TestAltOffset( void);

/* PEC... */

#define PECFile "PEC.DAT"
#define PECALT "PECALT"
#define PECAZ "PECAZ"
#define TXT "TXT"
#define DOT_TXT ".TXT"
#define PECSize 200

/* return inverse value from auto alt PEC sync line (parallel port pin 17) */
#define READ_AUTO_ALT_PEC_SYNC (AutoAltPECSyncSignal = (inportb( PPortAddrInOutNibble)& 8)>>3)
/* return inverse value from auto az PEC sync line (parallel port pin 15) */
#define READ_AUTO_AZ_PEC_SYNC (AutoAzPECSyncSignal = (inportb( PPortAddrInNibble)& 8)>>3)

struct PECWORK
{
	int TenthsArcsec;
	Flag Entry;
};

struct PECWORK* P;

typedef enum
{
	altaxis,
	azaxis
}ALTAZ_AXIS;

ALTAZ_AXIS Altaz_Axis;

int AltPECFileNameCount;
int AzPECFileNameCount;
char Fname[13];
int Ix;
int TenthsArcsec;
/* microsteps per one complete PEC revolution */
long MsPerPECArray;
/* microsteps per PEC index */
long MsPerPECIx;
/* offset between AccumMs%FullstepsPerPECArray and PEC array [0] */
struct AZInt PECIxOffset;
/* indexes to PEC array */
struct AZInt PECIx;
/* PEC to adjust Current coordinates by */
struct AZDouble PECToAdd;
/* fractional or decimal values of PECIx */
struct AZDouble DecimalPECIx;
/* direction of movement: if CW then PECIx is increasing, else decreasing */
struct AZFlag PECDir;
/* based on PECDir, the next PECIx */
struct AZInt NextPECIx;
/* arrays of PEC */
struct AZInt PECs[PECSize];

Flag AutoAltPECSyncSignal;
Flag LastAutoAltPECSyncSignal;
Flag AutoAzPECSyncSignal;
Flag LastAutoAzPECSyncSignal;

void InitPEC( void);
void SetPECVars( void);
void ZeroAltPEC( void);
void ZeroAzPEC( void);
void LoadPEC( void);
void SavePEC( char* Name);
void BuildAltPECFilename( void);
void BuildAzPECFilename( void);
void SynchPEC( void);
void SynchAltPEC( void);
void SynchAzPEC( void);
void GetPECIx( void);
void DisplayPECgraphically( void);
Flag AveragePECAnalysisFiles( Flag axis);

/* guide... */

#define GuideAltFile "GUIDEALT.DAT"
#define GuideAzFile "GUIDEAZ.DAT"
#define MaxGuideIx 600

struct GuideArray
{
	Byte PECIx;
	int TenthsArcsec;
}; 

struct AZGuideArray
{
	struct GuideArray A, Z;
};

typedef enum
{
	/* 0 for no processing underway */
	WritingToGuideArray = 1,
	ReadyToSaveGuideArray
}GUIDE_ARRAY_FLAG;

GUIDE_ARRAY_FLAG GuideArrayFlag;

Flag GuideFlag;

/* guiding speed in radians per clock tick */
double GuideRadTick;
/* accumulated guiding amounts in altitude and azimuth */
struct AZDouble AccumGuide;
struct AZInt GuideIx;
struct AZGuideArray Guides[MaxGuideIx];
/* direction of motion when writing PECIx's to guides[] */
struct AZFlag GuideDir;
struct AZInt PECIxCrossZero;
struct AZInt StartGuidesIxCrossZeroPECIx;
struct AZInt EndGuidesIxCrossZeroPECIx;

void InitGuide( void);
void SaveGuideAlt( void);
void SaveGuideAz( void);
void WriteToGuideArray( void);
void LoadGuideAlts( void);
void LoadGuideAzs( void);
void AnalyzeGuideAltArray( void);
void AnalyzeGuideAzArray( void);
Flag CreateAltPECFromGuideArray( void);
Flag CreateAzPECFromGuideArray( void);

/* IACA event... */

#define PUT_SCOPE if( IACA_Flag) PutScope( Current.Ra, Current.Dec)

long far* IACA_Ra_Decimal_Hrs_Ptr;
long far* IACA_Dec_Decimal_Degrees_Ptr;
long far* Scope_Ra_Decimal_Hrs_Ptr;
long far* Scope_Dec_Decimal_Degrees_Ptr;
double Conv_Factor;

void InitIACA( void);
void PutScope( double RaRad, double DecRad);
int NewIACA( void);
void GetIACA( double* RaRad, double* DecRad);
void TestIACA( void);

#endif
